@@ -2,6 +2,7 @@ from __future__ import unicode_literals
 
 import requests
 import time
+import mimetypes
 from bs4 import BeautifulSoup
 from datetime import datetime, timedelta
 from decimal import Decimal
@@ -11,7 +12,8 @@ from .exceptions import (
   WikiaException, ODD_ERROR_MESSAGE)
 from .util import cache, stdout_encode, debug
 
-
+# Generate all extensions from the OS
+mimetypes.init()
 API_URL = 'http://{lang}{sub_wikia}.wikia.com/api/v1/{action}'
 # URL used when browsing the wikia proper
 STANDARD_URL = 'http://{lang}{sub_wikia}.wikia.com/wiki/{page}'
@@ -78,7 +80,7 @@ def set_rate_limiting(rate_limit, min_wait=timedelta(milliseconds=50)):
 
 
 @cache
-def search(query, sub_wikia, results=10):
+def search(sub_wikia, query, results=10):
   '''
   Do a Wikia search for `query`.
 
@@ -132,7 +134,7 @@ def random(pages=1):
 
 
 @cache
-def summary(title, sub_wikia, chars=500, redirect=True):
+def summary(sub_wikia, title, chars=500, redirect=True):
   '''
   Plain text summary of the page from the sub-wikia.
 
@@ -140,7 +142,7 @@ def summary(title, sub_wikia, chars=500, redirect=True):
 
   Keyword arguments:
 
-  * chars - if set, return only the first `chars` characters (actual text retu
+  * chars - if set, return only the first `chars` characters (limit is 500)
   * auto_suggest - let Wikia find a valid page title for the query
   * redirect - allow redirection without raising RedirectError
   '''
@@ -367,11 +369,9 @@ class WikiaPage(object):
     and other data.
     '''
     if not getattr(self, '_content', False):
-      # First get the plaintext summary
       query_params = {
         'action': "Articles/AsSimpleJson?/",
         'id': self.pageid,
-        'ids': self.pageid, # Because for some reason the API uses both "id" and "ids"
         'sub_wikia': self.sub_wikia,
         'lang': LANG
       }
@@ -380,11 +380,6 @@ class WikiaPage(object):
       self._content = "\n".join(segment['text'] for section in request['sections']
                                                 for segment in section['content']
                                                 if segment['type'] == "paragraph")
-      # Then get the revision id
-      query_params['action'] = "Articles/Details?/"
-      request = _wiki_request(query_params)
-      self._revision_id = request['items'][str(self.pageid)]['revision']['id']
-
     return self._content
 
   @property
@@ -400,8 +395,14 @@ class WikiaPage(object):
     '''
 
     if not getattr(self, '_revid', False):
-      # fetch the content (side effect is loading the revid)
-      self.content
+      query_params = {
+        'action': "Articles/Details?/",
+        'ids': self.pageid,
+        'sub_wikia': self.sub_wikia,
+        'lang': LANG
+      }
+      request = _wiki_request(query_params)
+      self._revision_id = request['items'][str(self.pageid)]['revision']['id']
 
     return self._revision_id
 
@@ -410,22 +411,8 @@ class WikiaPage(object):
     '''
     Plain text summary of the page.
     '''
-
     if not getattr(self, '_summary', False):
-      query_params = {
-        'action': 'Articles/Details?/',
-        'query': self.title,
-        'sub_wikia': self.sub_wikia,
-        'lang': LANG,
-      }
-      if not getattr(self, 'title', None) is None:
-         query_params['titles'] = self.title
-      else:
-         query_params['pageids'] = self.pageid
-
-      request = _wiki_request(query_params)
-      self._summary = request['query']['pages'][self.pageid]['extract']
-
+      self._summary = summary(self.sub_wikia, self.title)
     return self._summary
 
   @property
@@ -435,16 +422,37 @@ class WikiaPage(object):
     '''
 
     if not getattr(self, '_images', False):
+      # Get the first round of images
       query_params = {
         'action': "Articles/AsSimpleJson?/",
-        'id': self.pageid,
+        'id': str(self.pageid),
         'sub_wikia': self.sub_wikia,
         'lang': LANG,
       }
       request = _wiki_request(query_params)
-      self._images = [section['images'][0]['src'] for section in request["sections"]
+      images = [section['images'][0]['src'] for section in request["sections"]
                      if section['images']]
 
+      # Get the second round of images
+      # This time, have to use a different API call
+      query_params.update({'action': "Articles/Details?/",
+                           'titles': self.title}) # This helps reduce redirects
+      request = _wiki_request(query_params)
+      images.append(request["items"][str(self.pageid)]["thumbnail"])
+
+      # A little URL manipulation is required to get the full sized version
+      # So we have to generate all the possible image extensions from mimetype
+      image_extensions = ["." + ext.split("/")[1]
+                         for ext in mimetypes.types_map.values()
+                         if ext.split("/")[0] == "image"]
+      # Remove the filler around the image url that reduces the size
+      for index, image in enumerate(images):
+        for extension in image_extensions:
+          if extension in image:
+            image = "".join(image.partition(extension)[:2])
+            images[index] = image.replace("/thumb/", "/")
+            break
+      self._images = images
     return self._images
 
   @property
@@ -524,17 +532,14 @@ def languages():
   Returns: dict of <prefix>: <local_lang_name> pairs. To get just a list of prefixes,
   use `wikia.languages().keys()`.
   '''
-  response = _wiki_request({
-    'meta': 'siteinfo',
-    'siprop': 'languages'
-  })
-
-  languages = response['query']['languages']
-
-  return {
-    lang['code']: lang['*']
-    for lang in languages
+  query_params = {
+    'action': "WAM/WAMLanguages?/",
+    'timestamp': time.time(), # Uses the UNIX timestamp to determine available LANGs
+    'sub_wikia': '',
+    'lang': LANG
   }
+  request = _wiki_request(query_params)
+  return response['languages']
 
 def _wiki_request(params):
   '''
@@ -570,8 +575,8 @@ def _wiki_request(params):
   except ValueError:
     raise WikiaError("Your request to the url \"{url}\" with the paramaters"
                      "\"{params}\" returned data in a format other than JSON."
-                     "Please check your input data.").format(url=api_url,
-                                                             params=params)
+                     "Please check your input data.".format(url=api_url,
+                                                             params=params))
   # If we got a json response, then we know the format of the input was correct
   if "exception" in r:
     details, message, error_code= r['exception'].values()
